<?php
/*
	$Project: Task Scheduler $
	$Author: karapuz team <support@ka-station.com> $

	$Version: 3.0.3.8 $ ($Revision: 139 $)
*/

// this namespace must be used for all tasks
namespace extension\ka_extensions\tasks;

// any task class consists of "Model" and "task class (similar to the file name)"
// other formats are not allowed
//
class ModelSendLog extends \KaModel {

	protected $file_to_delete = '';

	public function requestSchedulerOperations() {
	
		$ret = array(
			'send_log' => 'Send error Log to admin'
		);
	
		return $ret;
	}

	/*
		$operation - operation name (string)
		$params    - array of parameters
		$stat      - array of returned strings for displaying results of the operation
		
		RETURNS:
			$return - text code
				'finished'      - operation is complete
				'not_finished'  - still working (additional calls needed)
	*/
	public function runSchedulerOperation($operation, $params, &$stat) {
	
			$return_code = 'finished';
			
			/* put your code below this line */		

			$result = $this->sendLog();
			
			// return statistics
			//
			if (empty($result['sent'])) {
				$stat = array(
					'msg' => 'No new records',
				);
			} else {
				$stat = array(
					'msg'  => 'File was sent.',
					'size' => $result['filesize'] . ' bytes'
				);
			}
			
			return $return_code;
			
	}
	
	
	protected function sendLog() {

		$return = array(
			'sent' => false,
			'filesize' => 0,
		);
		
		$file = DIR_LOGS . $this->config->get('config_error_filename');
		
		if (!file_exists($file)) {
			return $return;
		}
		
		$last_change_time = max(filectime($file), filemtime($file));
		
		if (time() - $last_change_time > 3600*24) {
			return $return;
		}

		$log = file_get_contents($file);
		if (empty($log)) {
			return $return;
		}
		
		$ka_mail = new \KaMail($this->registry);
		$ka_mail->data['error_log'] = $log;
	
		$email = $this->config->get('config_email');

		if (defined('ADMIN_EMAIL')) {
			$email = ADMIN_EMAIL;
		}	

		$tpl = 'extension/ka_extensions/ka_scheduler/mail/send_log';
		
		$ka_mail->send($email, $email,
			$this->language->get('New Log Records at ' .$this->config->get('config_name')), $tpl
		);

		$return['sent'] = true;
		$return['filesize'] = filesize($file);

		$this->file_to_delete = $file;
		
		return $return;
	}
	
	public function __destruct() {
		if (!empty($this->file_to_delete)) {
			rename($this->file_to_delete, $this->file_to_delete . '.' . date('Y-m-d'));
		}
	}
}

// you have to specify a full class name alias for the short class name
//
class_alias(__NAMESPACE__ . '\ModelSendLog', 'ModelExtensionKaExtensionsTasksSendLog');