<?php
/*
	Project: Task Scheduler
	Author : karapuz <support@ka-station.com>

	Version: 3 ($Revision: 139 $)

	
*/
class ModelUpdateCurrencies extends \KaModel {

	public function getModuleName() {
		$name = 'Read and update currency rates from EU bank (once a day)';
		return $name;
	}

	/*
		This function is optional
	*/
	public function requestSchedulerOperations() {
	
		$ret = array(
			'update_currencies' => 'Update Currencies'
		);
	
		return $ret;
	}

	/*
		This function is optional
	*/
	public function requestSchedulerOperationParams($op) {
	
		$ret = array(
/*		
			'page_url' => array(
				'title' => 'Page URL',
				'type'  => 'text',
				'required' => true,
			),
*/			
		);
		
		return $ret;
	}

	/*
		This function is mandatory
		
		$operation - operation name (string)
		$params    - array of parameters
		$stat      - array of returned strings for displaying results of the operation
		
		RETURNS:
			$return - text code (
					'finished'      - operation is complete
					'not_finished'  - still working (additional calls needed)
					)
			
			$return - array(
					'result'        - text code (finished/not_finished)
					'next_task_id'  - task_id to request on completion
	*/
	public function runSchedulerOperation($operation, $params, &$stat) {
	
			$return_code = 'finished';
			
			/* put your code below this line */		

			$updated_currencies = $this->updateCurrencies();
			
			// return statistics
			//
			if (!empty($updated_currencies)) {
				$stat = array(
					'Updated currencies' => implode(', ', $updated_currencies),
				);
			} else {
				$stat = array(
					'Last error' => $this->getLastError(),
				);
			}
			
			return $return_code;
	}

	
	protected function updateCurrencies() {

		$return = array();
		
		$primary = $this->config->get('config_currency');
		$this->load->model('localisation/currency');
		$currencies = array_keys($this->model_localisation_currency->getCurrencies());

		// retreive currency rates for EUR currency from European bank
		//
		$url  = "https://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml";
		$xml  = file_get_contents($url);
		$data = simplexml_load_string($xml);
		
		if (empty($data)) {
			$this->lastError = "XML Data cannot be read";						
			return false;
		}

		$rates         = array();
		$reversal_rate = 0;
		
		// find a reversal rate for EUR/Primary currency
		//
		foreach ($data->Cube->Cube->Cube as $attr) {
		
			$res      = $attr->attributes();
			$currency = (string)$res['currency'];
			$rate     = (float)$res['rate'];

			if (!in_array($currency, $currencies))
				continue;
				
			$rates[$currency] = $rate;
			if ($currency == $primary) {
				$reversal_rate = 1 / $rate;
			}
		}
		$rates['EUR']  = 1;		
		
		if (empty($reversal_rate)) {
			$this->lastError = "Primary currency was not found in rates";
			return false;
		}
		
		// update rates in the database
		//
		$return = array();
		foreach ($rates as $currency => $rate) {
			$value = $rate * $reversal_rate;
			if ($currency == $primary) {
				$value = 1;
			}

			$this->db->query("UPDATE " . DB_PREFIX . "currency SET value = '" . (float)$value . "', date_modified = '" .  $this->db->escape(date('Y-m-d H:i:s')) . "' WHERE code = '" . $this->db->escape($currency) . "'");
			$return[] = $currency;
		}
		
		$this->cache->delete('currency');
		
		return $return;
	}
}

class_alias(__NAMESPACE__ . '\ModelUpdateCurrencies', 'ModelExtensionKaExtensionsTasksUpdateCurrencies');